Package: toxpiR
Type: Package
Title: Create ToxPi Prioritization Models
Version: 1.3.2
Authors@R: 
    c(person("Jonathon F", "Fleming",
             role = c("aut","cre"),
             email = "jffleming0129@gmail.com",
             comment = c(ORCID = "0000-0003-2447-3139")),
      person("Dayne L", "Filer", 
             role = c("aut", "fnd"), 
             email = "dayne.filer@gmail.com",
             comment = c(ORCID = "0000-0002-3443-5315")),
      person("Dillon T", "Lloyd",
             role = "aut"),
      person("Preethi", "Thunga",
             role = "aut",
             comment = c(ORCID = "0000-0001-5447-0129")),
      person("Skylar W", "Marvel",
             role = "aut"),
      person("Alison A", "Motsinger-Reif",
             role = c("fnd"),
             comment = c(ORCID = "0000-0003-1346-2493")),
      person("David M", "Reif",
             role = c("aut", "fnd"),
             email = "reif.david@gmail.com",
             comment = c(ORCID = "0000-0001-7815-6767")))
Description: 
  Enables users to build 'ToxPi' prioritization models and provides 
  functionality within the grid framework for plotting ToxPi graphs.
  'toxpiR' allows for more customization than the 'ToxPi GUI' 
  (<https://toxpi.github.io/>) and integration into existing workflows for greater 
  ease-of-use, reproducibility, and transparency.
  toxpiR package behaves nearly identically to the GUI; the package 
  documentation includes notes about all differences.
  The vignettes download example files from 
  <https://github.com/ToxPi/ToxPi-example-files>.
Imports: grDevices, methods, S4Vectors, grid, rlang, stats,
        BiocGenerics, tidyr, utils, ggplot2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 4.0)
Suggests: rmarkdown, knitr, testthat (>= 3.0.0), covr, DBI
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
License: GPL (>= 3)
URL: https://github.com/ToxPi/toxpiR, https://toxpi.github.io/toxpiR/
BugReports: https://github.com/ToxPi/toxpiR/issues
NeedsCompilation: no
Packaged: 2026-05-01 15:59:04 UTC; flemingjf
Author: Jonathon F Fleming [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2447-3139>),
  Dayne L Filer [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-3443-5315>),
  Dillon T Lloyd [aut],
  Preethi Thunga [aut] (ORCID: <https://orcid.org/0000-0001-5447-0129>),
  Skylar W Marvel [aut],
  Alison A Motsinger-Reif [fnd] (ORCID:
    <https://orcid.org/0000-0003-1346-2493>),
  David M Reif [aut, fnd] (ORCID:
    <https://orcid.org/0000-0001-7815-6767>)
Maintainer: Jonathon F Fleming <jffleming0129@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-01 16:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:11:58 UTC; unix
