# Generated by roxygen2: do not edit by hand

S3method(editDetails,pieGrob)
export(TxpModel)
export(TxpModelList)
export(TxpResultList)
export(TxpSlice)
export(TxpSliceList)
export(TxpTransFunc)
export(TxpTransFuncList)
export(as.TxpModelList)
export(as.TxpResultList)
export(as.TxpSliceList)
export(as.TxpTransFuncList)
export(grid.pieGridGrob)
export(grid.pieGrob)
export(pieGridGrob)
export(pieGrob)
export(txpCalculateScores)
export(txpExportGui)
export(txpImportGui)
exportClasses(TxpModel)
exportClasses(TxpModelList)
exportClasses(TxpResult)
exportClasses(TxpResultList)
exportClasses(TxpResultParam)
exportClasses(TxpSlice)
exportClasses(TxpSliceList)
exportClasses(TxpTransFunc)
exportClasses(TxpTransFuncList)
exportMethods("[")
exportMethods("names<-")
exportMethods("txpIDs<-")
exportMethods("txpSlices<-")
exportMethods("txpTransFuncs<-")
exportMethods("txpValueNames<-")
exportMethods("txpWeights<-")
exportMethods(as.data.frame)
exportMethods(duplicated)
exportMethods(length)
exportMethods(merge)
exportMethods(names)
exportMethods(plot)
exportMethods(sort)
exportMethods(txpCalculateScores)
exportMethods(txpIDs)
exportMethods(txpMissing)
exportMethods(txpModel)
exportMethods(txpRanks)
exportMethods(txpResultParam)
exportMethods(txpScores)
exportMethods(txpSliceScores)
exportMethods(txpSlices)
exportMethods(txpTransFuncs)
exportMethods(txpValueNames)
exportMethods(txpWeights)
import(ggplot2)
import(grid)
import(methods)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(S4Vectors,character_OR_NULL)
importFrom(BiocGenerics,as.data.frame)
importFrom(BiocGenerics,duplicated)
importFrom(BiocGenerics,sort)
importFrom(BiocGenerics,updateObject)
importFrom(S4Vectors,List)
importFrom(S4Vectors,classNameForDisplay)
importFrom(S4Vectors,coolcat)
importFrom(S4Vectors,new2)
importFrom(S4Vectors,setValidity2)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,extendrange)
importFrom(grDevices,rgb)
importFrom(rlang,is_integerish)
importFrom(rlang,is_named)
importFrom(rlang,is_scalar_character)
importFrom(rlang,is_scalar_integerish)
importFrom(rlang,is_scalar_logical)
importFrom(stats,sd)
importFrom(tidyr,separate)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,type.convert)
importFrom(utils,write.table)
